(require 'seq)
(require 'subr-x)

(defgroup guix-profiles nil
  "Settings for guix profile management")

(defcustom guix-home-dir
  ""
  "The directory in which to look for the user's guix installation, typically the users homedir."
  :type '(string)
  :group 'guix-profiles)

(defcustom guix-available-profiles
  '()
  "List of all available profiles, each profile is represented by a cons pair of the profile name and a plist containing at least the attribute :profile-file which should be a string pointing to the profile location"
  :type '(repeat (cons string (plist)))
  :group 'guix-profiles)


(defcustom guix-active-profiles
  '()
  "The names of the profiles that are currently active."
  :type '(repeat string)
  :group 'guix-profiles)

(defun guix-profile-or-name->profile (profile-or-name)
  (if (stringp profile-or-name)
      (assoc profile-or-name guix-available-profiles)
    profile-or-name))

(defun guix-profile->profile-file (profile-or-name)
  (and-let* ((profile (guix-profile-or-name->profile profile-or-name))
             (profile-plist (cdr profile))
             (profile-file (plist-get profile-plist :profile-file)))
    profile-file))

(defun guix-profile->manifest-file (profile-or-name)
  (and-let* ((profile (guix-profile-or-name->profile profile-or-name))
             (profile-plist (cdr profile))
             (manifest-file (plist-get profile-plist :manifest-file)))
    manifest-file))

(defun guix-enable-profile (profile)
  (interactive (list (ido-completing-read "Profile: " (mapcar #'car guix-available-profiles))))
  (setq guix-active-profiles (cons profile guix-active-profiles))
  (guix-set-paths guix-active-profiles))

(defun guix-disable-profile (profile)
  (interactive (list (ido-completing-read "Profile: " guix-active-profiles)))
  (setq guix-active-profiles (remove profile guix-active-profiles))
  (guix-set-paths guix-active-profiles))

(defun fold (func acc lst)
  (if lst
      (fold func
            (apply func (car lst) acc '())
            (cdr lst))
    acc))

(defun cons* (&rest args)
  (apply
   #'apply
   #'list
   args))

(defun guix-profiles->environment-vars (profiles)
  (shell-command-to-string
   (string-join (apply #'list
                       "guix"
                       "package"
                       "--search-paths"
                       (fold (lambda (element accumulator)
                               (cons "-p" (cons element accumulator)))
                             nil
                             (cons* "/run/current-system/profile"
                                    (format "%s/.config/guix/current" guix-home-dir)
                                    (format "%s/.guix-profile" guix-home-dir)
                                    (mapcar #'guix-profile->profile-file
                                            profiles))))
                " ")))

(defun guix-path-file->key-value (str)
  (mapcar (lambda (line)
              (let* ((lst (split-string line "="))
                     (var (string-remove-prefix "export " (car lst)))
                     (val (string-trim (string-join (cdr lst) "=") "\"" "\"")))
                (cons var val)))
          (split-string (string-trim-right str "\n")
                        "\n")))

(defun guix-set-paths (profiles)
  (interactive (list guix-active-profiles))
  (setq exec-path ; TODO: properly handle escaped :'s in $PATH
        (cons "/run/setuid-programs"
              (split-string
               (alist-get "PATH"
                          (guix-path-file->key-value (guix-profiles->environment-vars profiles))
                          '()
                          nil
                          #'string=)
               ":")))
  (mapcar (lambda (pair)
            (setenv (car pair)
                    (cdr pair)))
          (guix-path-file->key-value (guix-profiles->environment-vars profiles)))
  (setenv "PATH" (format "/run/setuid-programs/:%s" (getenv "PATH"))))

(provide 'guix-profiles)
